@@ -15,12 +15,14 @@ from noc.lib.url import URL
 from noc.lib.fileutils import rewrite_when_differ,read_file,is_differ,in_dir
 from noc.lib.validators import is_int
 from noc.cm.vcs import vcs_registry
-import os,datetime,stat,logging,random,types
-from noc.sa.models import Activator,AdministrativeDomain,ObjectGroup,ManagedObject
-from noc.main.menu import Menu
+import os,datetime,stat,logging,random,types,time
+from noc.sa.models import Activator,AdministrativeDomain,ManagedObject
 from noc.sa.protocols.sae_pb2 import *
-from noc.main.search import SearchResult
+from noc.lib.search import SearchResult
 from noc.main.models import NotificationGroup
+from noc.lib.app.site import site
+from noc.lib.fields import AutoCompleteTagsField
+from tagging.models import TaggedItem
 
 profile_registry.register_all()
 vcs_registry.register_all()
@@ -35,12 +37,16 @@ class ObjectNotify(models.Model):
         verbose_name_plural="Object Notifies"
     type=models.CharField("Type",max_length=16,choices=OBJECT_TYPE_CHOICES)
     administrative_domain=models.ForeignKey(AdministrativeDomain,verbose_name="Administrative Domain",blank=True,null=True)
-    group=models.ForeignKey(ObjectGroup,verbose_name="Group",blank=True,null=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     notify_immediately=models.BooleanField("Notify Immediately")
     notify_delayed=models.BooleanField("Notify Delayed")
     notification_group=models.ForeignKey(NotificationGroup,verbose_name="Notification Group")
     def __unicode__(self):
-        return "(%s,%s,%s,%s)"%(self.type,self.administrative_domain,self.group,self.notification_group)
+        return "(%s,%s,[%s],%s)"%(self.type,self.administrative_domain,self.tags,self.notification_group)
+    
+    def get_absolute_url(self):
+        return site.reverse("cm:objectnotify:change",self.id
+        )
 
 #
 class Object(models.Model):
@@ -93,6 +99,11 @@ class Object(models.Model):
         return self.vcs.in_repo(self.repo_path)
     in_repo=property(_in_repo)
     #
+    #
+    #
+    def status(self):
+        return {True:"Ready",False:"Waiting"}[self.in_repo]
+    #
     # If "data" differs from object's content in the repository
     # Write "data" to file and commit
     #
@@ -122,11 +133,6 @@ class Object(models.Model):
             self.vcs.rm(self.path)
         super(Object,self).delete()
 
-    def view_link(self):
-        return "<A HREF='/cm/view/%s/%d/'>View</A>"%(self.repo_name,self.id)
-    view_link.short_description="View"
-    view_link.allow_tags=True
-
     def _revisions(self):
         return self.vcs.log(self.repo_path)
     revisions=property(_revisions)
@@ -138,6 +144,10 @@ class Object(models.Model):
             if r.revision==rev:
                 return r
         raise Exception("Not found")
+    # Return object's current revision
+    def current_revision(self):
+        return self.vcs.get_current_revision(self.repo_path)
+    current_revision=property(current_revision)
     
     def diff(self,rev1,rev2):
         return self.vcs.diff(self.repo_path,rev1,rev2)
@@ -169,6 +179,12 @@ class Object(models.Model):
     def _verbose_name_plural(self):
         return self._meta.verbose_name_plural
     verbose_name_plural=property(_verbose_name_plural)
+    #
+    #
+    #
+    def verbose_name(self):
+        return self._meta.verbose_name
+    verbose_name=property(verbose_name)
     
     def get_notification_groups(self,immediately=False,delayed=False):
         q=Q(type=self.repo_name)
@@ -222,25 +238,28 @@ class Object(models.Model):
     @classmethod
     def search(cls,user,query,limit):
         for o in [o for o in cls.objects.all() if o.repo_path and o.has_access(user)]:
-            data=o.data
-            if query in o.repo_path: # If repo_path matches
-                yield SearchResult(
-                    url="/cm/view/%s/%d/"%(o.repo_name,o.id),
-                    title="CM: "+unicode(o),
-                    text=unicode(o),
-                    relevancy=1.0, # No weighted search yes
-                    )                
-            elif data and query in data: # Dumb substring search in config
-                idx=data.index(query)
-                idx_s=max(0,idx-100)
-                idx_e=min(len(data),idx+len(query)+100)
-                text=data[idx_s:idx_e]
-                yield SearchResult(
-                    url="/cm/view/%s/%d/"%(o.repo_name,o.id),
-                    title="CM: "+unicode(o),
-                    text=text,
-                    relevancy=1.0, # No weighted search yes
-                    )
+            try:
+                data=o.data
+                if query in o.repo_path: # If repo_path matches
+                    yield SearchResult(
+                        url=("cm:%s:view"%o.repo_name,o.id),
+                        title="CM: "+unicode(o),
+                        text=unicode(o),
+                        relevancy=1.0, # No weighted search yes
+                        )                
+                elif data and query in data: # Dumb substring search in config
+                    idx=data.index(query)
+                    idx_s=max(0,idx-100)
+                    idx_e=min(len(data),idx+len(query)+100)
+                    text=data[idx_s:idx_e]
+                    yield SearchResult(
+                        url=("cm:%s:view"%o.repo_name,o.id),
+                        title="CM: "+unicode(o),
+                        text=text,
+                        relevancy=1.0, # No weighted search yes
+                        )
+            except UnicodeDecodeError: # Skip unicode errors in configs
+                pass
 ##
 ## Config
 ##
@@ -257,27 +276,32 @@ class Config(Object):
     
     def pull(self,sae):
         def pull_callback(result=None,error=None):
+            dt=int(time.time()-t0)
             if error:
                 if error.code==ERR_OVERLOAD:
                     timeout=config.getint("cm","timeout_overload")
+                    status="ERR_OVERLOAD"
                 elif error.code==ERR_DOWN:
                     timeout=config.getint("cm","timeout_down")
+                    status="ERR_DOWN"
                 else:
                     timeout=config.getint("cm","timeout_error")
+                    status="ERR_TIMEOUT"
                 variation=config.getint("cm","timeout_variation")
                 timeout+=random.randint(-timeout/variation,timeout/variation) # Add jitter to avoid blocking by dead task
                 self.next_pull=datetime.datetime.now()+datetime.timedelta(seconds=timeout)
                 self.save()
+                logging.info("Config.pull(): object_id=%d, object_name=%s, status=%s, time_elapsed=%d, timeout=%d"%(
+                    self.managed_object.id,self.managed_object.name,status,dt,timeout))
                 return
             if self.pull_every:
                 self.next_pull=datetime.datetime.now()+datetime.timedelta(seconds=self.pull_every)
                 self.save()
+                logging.info("Config.pull(): object_id=%d, object_name=%s, status=%s, time_elapsed=%d, timeout=%d"%(
+                    self.managed_object.id,self.managed_object.name,"OK",dt,self.pull_every))
             self.write(result)
+        t0=time.time()
         sae.script(self.managed_object,"%s.get_config"%self.managed_object.profile_name,pull_callback)
-    def scripts_link(self):
-        return "<A HREF='/sa/%d/scripts/'>Scripts</A>"%(self.id)
-    scripts_link.short_description="Scripts"
-    scripts_link.allow_tags=True
     ##
     ## Access control
     ##
@@ -288,30 +312,8 @@ class Config(Object):
     def queryset(cls,user):
         if user.is_superuser:
             return cls.objects.all()
-        # Build query
-        r=[]
-        p=[]
-        for a in user.useraccess_set.all():
-            if a.administrative_domain is None and a.group is None: # Full access
-                return cls.objects.all()
-            rr=[]
-            pp=[]
-            if a.administrative_domain:
-                rr.append("(sa_managedobject.administrative_domain_id=%s)")
-                pp.append(a.administrative_domain.id)
-            if a.group:
-                rr.append("(id IN (SELECT managedobject_id FROM sa_managedobject_groups WHERE objectgroup_id=%s))")
-                pp.append(a.group.id)
-            if len(rr)==1: # Single clause
-                r+=rr
-            else: # AND together
-                r+=["(%s AND %s)"%(rr[0],rr[1])]
-            p+=pp
-        if not r: # No access
-            return cls.objects.extra(where=["0=1"]) # Return empty queryset
-        where=" OR ".join(r)
-        where="(managed_object_id IN (SELECT id FROM sa_managedobject WHERE %s))"%where
-        return cls.objects.extra(where=[where],params=p)
+        else:
+            return cls.objects.filter(managed_object__in=ManagedObject.user_objects(user))
     
     def get_notification_groups(self,immediately=False,delayed=False):
         q=Q(type=self.repo_name)
@@ -320,7 +322,13 @@ class Config(Object):
         if delayed:
             q&=Q(notify_delayed=True)
         q&=(Q(administrative_domain__isnull=True)|Q(administrative_domain=self.managed_object.administrative_domain))
-        q&=(Q(group__isnull=True)|Q(group__in=self.managed_object.groups.all))
+        if self.managed_object.tags:
+            tagged=TaggedItem.objects.get_union_by_model(ObjectNotify,self.managed_object.tags).values_list("id",flat=True)
+            if tagged:
+                q&=(Q(tags__isnull=True)|Q(tags="")|Q(id__in=tagged))
+            else:
+                q&=(Q(tags__isnull=True)|Q(tags=""))
+        print q
         return set([n.notification_group for n in ObjectNotify.objects.filter(q)])
 
     def write(self,data):
@@ -331,6 +339,13 @@ class Config(Object):
                 r+=["==[ %s ]========================================\n%s"%(d["name"],d["config"])]
             data="\n".join(r)
         super(Config,self).write(data)
+    ##
+    ##
+    ##
+    def change_link(self):
+        return "<a href='%s'>Change</a>"%site.reverse("cm:config:change",self.id)
+    change_link.short_description="Change"
+    change_link.allow_tags=True
 
 ##
 ## PrefixList
@@ -490,18 +505,3 @@ class RPSL(Object):
     
     @classmethod
     def global_push(cls): pass
-##
-## Application Menu
-##
-class AppMenu(Menu):
-    app="cm"
-    title="Configuration Management"
-    items=[
-        ("Config",      "/admin/cm/config/",     "cm.change_config"),
-        ("DNS Objects", "/admin/cm/dns/",        "cm.change_dns"),
-        ("Prefix Lists","/admin/cm/prefixlist/", "cm.change_prefixlist"),
-        ("RPSL Objects","/admin/cm/rpsl/",       "cm.change_rpsl"),
-        ("Setup",[
-            ("Object Notifies","/admin/cm/objectnotify/", "cm.change_objectnotify"),
-        ])
-    ]
