@@ -27,10 +27,24 @@ class VCS(noc.cm.vcs.VCS):
                 date,r=date.split("-",1)
             revs+=[noc.cm.vcs.Revision(rev,datetime.datetime.fromtimestamp(float(date)))]
         return revs
+        
+    def get_current_revision(self,path):
+        l=self.cmd_out(["log","-l","1","--template","{rev} {date}",path])
+        if not l:
+            return None
+        rev,date=l.split(" ")
+        if "-" in date:
+            date,r=date.split("-",1)
+        return noc.cm.vcs.Revision(rev,datetime.datetime.fromtimestamp(float(date)))
+        
     def diff(self,path,rev1,rev2):
         return self.cmd_out(["diff","-r%s:%s"%(rev1.revision,rev2.revision),path])
-    def get_revision(self,path,rev):
-        return self.cmd_out(["cat","-r%s"%rev.revision,path])
+    def get_revision(self,path,rev=None):
+        args=["cat"]
+        if rev is not None:
+            args+=["-r%s"%rev.revision]
+        args+=[path]
+        return self.cmd_out(args)
     def mv(self,f,t):
         self.cmd(["mv",f,t])
         self.commit(f,"mv")
