@@ -13,11 +13,11 @@ from noc.lib.validators import check_rd,check_cidr,is_cidr,is_ipv4
 from noc.lib.tt import tt_url
 from noc.peer.models import AS
 from noc.vc.models import VC
-from noc.lib.fields import CIDRField
+from noc.lib.fields import CIDRField,AutoCompleteTagsField
 from noc.lib.ip import int_to_address,bits_to_int,wildcard,broadcast,address_to_int,generate_ips,prefix_to_size
-from noc.main.menu import Menu
-from noc.main.search import SearchResult
-from noc.main.middleware import get_user
+from noc.lib.search import SearchResult
+from noc.lib.middleware import get_user
+from noc.lib.app import site
 ##
 ##
 ##
@@ -28,8 +28,6 @@ class VRFGroup(models.Model):
     name=models.CharField("VRF Group",unique=True,max_length=64)
     unique_addresses=models.BooleanField("Unique addresses in group")
     description=models.CharField("Description",blank=True,null=True,max_length=128)
-    def __str__(self):
-        return self.name
     def __unicode__(self):
         return unicode(self.name)
 ##
@@ -44,14 +42,26 @@ class VRF(models.Model):
     rd=models.CharField("rd",unique=True,max_length=21) # validator_list=[check_rd],
     description=models.CharField("Description",blank=True,null=True,max_length=128)
     tt=models.IntegerField("TT",blank=True,null=True)
-    def __str__(self):
-        if self.rd=="0:0":
-            return "global"
-        return self.rd
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
         if self.rd=="0:0":
             return u"global"
         return self.rd
+    
+    def get_absolute_url(self):
+        return site.reverse("ip:vrf:change",self.id)
+        
+    @classmethod
+    def get_global(self):
+        return VRF.objects.get(rd="0:0")
+    
+    def save(self):
+        super(VRF,self).save()
+        # Create 0.0.0.0/0 prefix if does not exists
+        try:
+            IPv4Block.objects.get(vrf=self,prefix="0.0.0.0/0")
+        except IPv4Block.DoesNotExist:
+            IPv4Block(vrf=self,prefix="0.0.0.0/0",description="Root",asn=AS.default_as()).save()
         
     def prefix(self,top="0.0.0.0/0"):
         assert is_cidr(top)
@@ -72,6 +82,13 @@ class VRF(models.Model):
         c=connection.cursor()
         c.execute("SELECT id FROM %s WHERE vrf_id=%d and ip<<='%s' order by ip"%(IPv4Address._meta.db_table,self.id,top))
         return [IPv4Address.objects.get(id=x[0]) for x in c.fetchall()]
+    ##
+    ## Find longest ipv4 block holding address
+    ##
+    def find_ipv4block(self,address):
+        c=connection.cursor()
+        c.execute("SELECT id FROM %s WHERE vrf_id=%d AND '%s' << prefix ORDER BY masklen(prefix) DESC LIMIT 1"%(IPv4Block._meta.db_table,self.id,address))
+        return IPv4Block.objects.get(id=c.fetchall()[0][0])
 ##
 ##
 ##
@@ -84,8 +101,6 @@ class IPv4BlockAccess(models.Model):
     vrf=models.ForeignKey(VRF,verbose_name="VRF")
     prefix=CIDRField("prefix")
     tt=models.IntegerField("TT",blank=True,null=True)
-    def __str__(self):
-        return "%s: %s(%s)"%(self.user,self.prefix,self.vrf)
     @classmethod
     def check_write_access(self,user,vrf,prefix):
         assert is_cidr(prefix)
@@ -108,14 +123,15 @@ class IPv4Block(models.Model):
     vrf=models.ForeignKey(VRF)
     asn=models.ForeignKey(AS)
     vc=models.ForeignKey(VC,verbose_name="VC",null=True,blank=True)
-    modified_by=models.ForeignKey(User,verbose_name="User")
-    last_modified=models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     tt=models.IntegerField("TT",blank=True,null=True)
     #allocated_till
-    def __str__(self):
-        return str(self.prefix)
     def __unicode__(self):
         return unicode(self.prefix)
+    
+    def get_absolute_url(self):
+        return site.reverse("ip:ipmanage:vrf_index",self.vrf.id,self.prefix)
+    
     def _parent(self):
         c=connection.cursor()
         c.execute("SELECT id FROM %s WHERE prefix >> '%s' AND vrf_id=%d ORDER BY masklen(prefix) DESC LIMIT 1"%(IPv4Block._meta.db_table,
@@ -293,10 +309,31 @@ class IPv4Block(models.Model):
                         relevancy=float(len(search))/float(l)
                     else:
                         relevancy=0.0
-                yield SearchResult(url="/ip/%d/%s/"%(r.vrf.id,r.prefix),
+                yield SearchResult(
+                    url=("ip:ipmanage:vrf_index",r.vrf.id,r.prefix),
                     title="IPv4 Block, VRF=%s, %s"%(r.vrf,r.prefix),
                     text=r.description,
                     relevancy=relevancy)
+    ##
+    ## Check user has bookmark on prefix
+    ##
+    def has_bookmark(self,user):
+        return IPv4BlockBookmark.has_bookmark(user,self)
+    ##
+    ## Set bookmark for user
+    ##
+    def set_bookmark(self,user):
+        IPv4BlockBookmark.set_bookmark(user,self)
+    ##
+    ## Remove bookmark for user
+    ##
+    def set_bookmark(self,user):
+        IPv4BlockBookmark.set_bookmark(user,self)
+    ##
+    ## Toggle bookmark for user
+    ##
+    def toggle_bookmark(self,user):
+        IPv4BlockBookmark.toggle_bookmark(user,self)
 ##
 ## IPv4 Address Range
 ##
@@ -467,15 +504,14 @@ class IPv4Address(models.Model):
     fqdn=models.CharField("FQDN",max_length=64)
     ip=models.IPAddressField("IP")
     description=models.CharField("Description",blank=True,null=True,max_length=64)
-    modified_by=models.ForeignKey(User,verbose_name="User")
-    last_modified=models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     tt=models.IntegerField("TT",blank=True,null=True)
     
     def __unicode__(self):
         return self.ip
-
-    def __str__(self):
-        return self.ip
+        
+    def get_absolute_url(self):
+        return site.reverse("ip:ipmanage:vrf_index",self.vrf.id,self.parent)
 
     def _parent(self):
         c=connection.cursor()
@@ -518,23 +554,54 @@ class IPv4Address(models.Model):
                     else:
                         r_description=0.0
                     relevancy=max(r_fqdn,r_description)
-                yield SearchResult(url="/ip/%d/%s/assign_address/"%(r.vrf.id,r.ip),
+                yield SearchResult(
+                    url=("ip:ipmanage:change_address",r.vrf.id,r.ip),
                     title="IPv4 Address, VRF=%s, %s (%s)"%(r.vrf,r.ip,r.fqdn),
                     text=r.description,
                     relevancy=relevancy)
 ##
-## Application Menu
+## User Bl
 ##
-class AppMenu(Menu):
-    app="ip"
-    title="Address Space Management"
-    items=[
-        ("Assigned addresses", "/ip/", "ip.change_ipv4block"),
-        ("Setup",[
-            ("VRF Groups",  "/admin/ip/vrfgroup/",        "ip.change_vrfgroup"),
-            ("VRFs",        "/admin/ip/vrf/",             "ip.change_vrf"),
-            ("Block Access","/admin/ip/ipv4blockaccess/", "ip.change_ipv4blockaccess"),
-            ("IPv4 Address Ranges","/admin/ip/ipv4addressrange/", "ip.change_ipv4addressrange"),
-            ]
-        )
-    ]
+class IPv4BlockBookmark(models.Model):
+    class Meta:
+        verbose_name="IPv4 Block Bookmark"
+        verbose_name="IPv4 Block Bookmarks"
+        unique_together=[("user","prefix")]
+    user=models.ForeignKey(User,verbose_name="User")
+    prefix=models.ForeignKey(IPv4Block,verbose_name="Prefix")
+    def __unicode__(self):
+        return u"Bookmark %s for %s"%(self.prefix.prefix,self.user.username)
+    ##
+    ## Check the prefix is bookmarked
+    ##
+    @classmethod
+    def has_bookmark(cls,user,prefix):
+        return cls.objects.filter(user=user,prefix=prefix).count()>0
+    ##
+    ## Get a list of user bookmarks
+    ##
+    @classmethod
+    def user_bookmarks(cls,user,vrf):
+        return [b.prefix for b in cls.objects.filter(user=user,prefix__vrf=vrf)]
+    ##
+    ## Set bookmark
+    ##
+    @classmethod
+    def set_bookmark(cls,user,prefix):
+        if not cls.has_bookmark(user,prefix):
+            cls(user=user,prefix=prefix).save()
+    ##
+    ## Remove bookmarks
+    ##
+    @classmethod
+    def remove_bookmark(cls,user,prefix):
+        cls.objects.filter(user=user,prefix=prefix).delete()
+    ##
+    ## Toggle bookmark
+    ##
+    @classmethod
+    def toggle_bookmark(cls,user,prefix):
+        if cls.has_bookmark(user,prefix):
+            cls.remove_bookmark(user,prefix)
+        else:
+            cls.set_bookmark(user,prefix)
