@@ -8,6 +8,9 @@
 from django.db import models
 from lib.ip import normalize_prefix
 import types,cPickle
+from django.contrib.admin.widgets import AdminTextInputWidget
+from tagging.fields import TagField
+from south.modelsinspector import add_introspection_rules
 ##
 ## CIDRField maps to PostgreSQL CIDR
 ##
@@ -88,11 +91,24 @@ class PickledField(models.Field):
     def db_type(self):
         return "TEXT"
     def to_python(self,value):
-        if not value:
-            return None
+        #if not value:
+        #    return None
         try:
             return cPickle.loads(str(value))
         except:
             return value
     def get_db_prep_value(self,value):
         return cPickle.dumps(value)
+##
+## Autocomplete tags fields
+##
+class AutoCompleteTagsField(TagField):
+    def formfield(self, **kwargs):
+        from noc.lib.widgets import AutoCompleteTags
+        defaults = {'widget': AutoCompleteTags}
+        defaults.update(kwargs)
+        if defaults['widget'] == AdminTextInputWidget:
+            defaults['widget']=AutoCompleteTags
+        return super(AutoCompleteTagsField,self).formfield(**defaults)
+##
+add_introspection_rules([],["^noc\.lib\.fields\."])
\ No newline at end of file
