@@ -54,7 +54,7 @@ class Registry(object):
                                 open(i_path,"w").close() # Create file
                     else:
                         mb=app+"."+".".join(dirpath.split(os.sep)[1:])
-                    for f in [f for f in filenames if f.endswith(".py")]:
+                    for f in [f for f in filenames if not f.startswith(".") and f.endswith(".py")]:
                         if f=="__init__.py":
                             f=""
                         else:
