@@ -7,31 +7,39 @@
 """
 from __future__ import with_statement
 from django.db import models
-from django.contrib.auth.models import User
-from noc.main.report import report_registry
-from noc.main.calculator import calculator_registry
-from noc.main.menu import Menu
+from django.contrib.auth.models import User,Group
 from noc.lib.fields import BinaryField
 from noc.lib.database_storage import DatabaseStorage as DBS
-import noc.main.search # Set up signal handlers
 import os,datetime,re,datetime,threading
 from noc.main.refbooks.downloaders import downloader_registry
-from noc.main.search import SearchResult
 from django.contrib import databrowse
 from django.db.models.signals import class_prepared,pre_save,pre_delete
 from noc.lib.fields import TextArrayField
-from noc.main.middleware import get_user
-from noc.settings import IS_WEB
+from noc.lib.middleware import get_user
+from noc import settings
 from noc.lib.timepattern import TimePattern as TP
 from noc.lib.timepattern import TimePatternList
 from noc.sa.interfaces.base import interface_registry
-
+from noc.lib.app.site import site
 ##
-## Databrowse register hook to intersept model creation
+## A hash of Model.search classmethods.
+## Populated by "class_prepared" signal listener
+## Model.search is a generator taking parameters (user,query,limit)
+## And yielding a SearchResults (ordered by relevancy)
 ##
-def register_databrowse_model(sender,**kwargs):
+search_methods={}
+##
+## Register new search handler if model has .search classmethod
+##
+def on_new_model(sender,**kwargs):
+    if hasattr(sender,"search"):
+        search_methods[getattr(sender,"search")]=None
     databrowse.site.register(sender)
-class_prepared.connect(register_databrowse_model)
+##
+## Attach to the 'class_prepared' signal
+## and on_new_model on every new model
+##
+class_prepared.connect(on_new_model)
 ##
 ## Exclude tables from audit
 ##
@@ -54,7 +62,10 @@ def audit_trail_save(sender,instance,**kwargs):
     #
     if instance.id:
         # Update
-        old=sender.objects.get(id=instance.id)
+        try:
+            old=sender.objects.get(id=instance.id)
+        except sender.DoesNotExist: # Protection for correct test fixtures loading
+            return
         message=[]
         operation="M"
         for f in sender._meta.fields:
@@ -84,7 +95,7 @@ def audit_trail_delete(sender,instance,**kwargs):
 ##
 ## Set up audit trail handlers
 ##
-if IS_WEB:
+if settings.IS_WEB:
     pre_save.connect(audit_trail_save)
     pre_delete.connect(audit_trail_delete)
 ##
@@ -92,8 +103,6 @@ if IS_WEB:
 ##
 downloader_registry.register_all()
 ##
-calculator_registry.register_all()
-##
 ## Audit Trail
 ##
 class AuditTrail(models.Model):
@@ -125,6 +134,67 @@ class AuditTrail(models.Model):
             body=message
         ).save()
 ##
+## Permissions
+## Populated by manage.py sync-perm
+##
+class Permission(models.Model):
+    class Meta:
+        verbose_name="Permission"
+        verbose_name_plural="Permissions"
+    name=models.CharField("Name",max_length=128,unique=True) # module:app:permission
+    users=models.ManyToManyField(User,related_name="noc_user_permissions")
+    groups=models.ManyToManyField(Group,related_name="noc_group_permissions")
+    def __unicode__(self):
+        return self.name
+    ##
+    ## Checks the user has permission directly or via group
+    ##
+    @classmethod
+    def has_perm(self,user,perm):
+        if not user.is_active:
+            return False
+        if user.is_superuser:
+            return True
+        p=Permission.objects.get(name=perm)
+        return bool(p.users.filter(id=user.id)) or bool(p.groups.filter(id__in=user.groups.all()))
+    ##
+    ## Return a set of user permissions
+    ##
+    @classmethod
+    def get_user_permissions(cls,user):
+        return set(user.noc_user_permissions.values_list("name",flat=True))
+    ##
+    ## Set user permissions to ``perm``. Perm must be set([])
+    ##
+    @classmethod
+    def set_user_permissions(cls,user,perms):
+        current=cls.get_user_permissions(user)
+        # Add new permissions
+        for p in perms-current:
+            Permission.objects.get(name=p).users.add(user)
+        # Revoke permissions
+        for p in current-perms:
+            Permission.objects.get(name=p).users.remove(user)
+    ##
+    ## Returns a set of group permissions
+    ##
+    @classmethod
+    def get_group_permissions(cls,group):
+        return set(group.noc_group_permissions.values_list("name",flat=True))
+    ##
+    ## Set group permissions
+    ##
+    @classmethod
+    def set_group_permissions(cls,group,perms):
+        current=cls.get_group_permissions(group)
+        # Add new permissions
+        for p in perms-current:
+            Permission.objects.get(name=p).groups.add(group)
+        # Revoke permissions
+        for p in current-perms:
+            Permission.objects.get(name=p).groups.remove(group)
+    
+##
 ## Languages
 ##
 class Language(models.Model):
@@ -196,6 +266,7 @@ class MIMEType(models.Model):
 ## pyRule
 ##
 class NoPyRuleException(Exception): pass
+rx_coding=re.compile(r"^#\s*-\*-\s*coding:\s*\S+\s*-\*-\s*$",re.MULTILINE)
 
 class PyRule(models.Model):
     class Meta:
@@ -205,6 +276,7 @@ class PyRule(models.Model):
     interface=models.CharField("Interface",max_length=64,choices=[(i,i) for i in interface_registry])
     description=models.TextField("Description")
     text=models.TextField("Text")
+    is_builtin=models.BooleanField("Is Builtin",default=False)
     changed=models.DateTimeField("Changed",auto_now=True,auto_now_add=True)
     # Compiled pyRules cache
     compiled_pyrules={}
@@ -213,6 +285,10 @@ class PyRule(models.Model):
     NoPyRule=NoPyRuleException
     def __unicode__(self):
         return self.name
+    # Check syntax
+    def save(self,**kwargs):
+        self.compile_text(unicode(self.text))
+        super(PyRule,self).save(**kwargs)
     # Returns an interface class
     def _interface_class(self):
         return interface_registry[self.interface]
@@ -220,18 +296,30 @@ class PyRule(models.Model):
     ##
     @classmethod
     def compile_text(self,text):
-        d={}
-        exec text.replace("\r\n","\n") in d
-        if "rule" not in d:
-            raise SyntaxError,"No 'rule' symbol defined"
-        rule=d["rule"]
+        # Built-in pyRule decorator
+        def pyrule(f):
+            f.is_pyrule=True
+            return f
+        # Inject @pyrule decorator into namespace
+        d={"pyrule":pyrule}
+        # Remove coding declarations and \r
+        text=rx_coding.sub("",text.replace("\r\n","\n"))
+        # Compile text
+        exec text in d
+        # Find marked pyrule
+        rules=[r for r in d.values() if hasattr(r,"is_pyrule") and r.is_pyrule]
+        if len(rules)<1:
+            raise SyntaxError("No @pyrule decorated symbol found")
+        if len(rules)!=1:
+            raise SyntaxError("More than one @pyrule deorated symbols found")
+        rule=rules[0]
         if not callable(rule):
-            raise SyntaxError,"'rule' is not callable"
+            raise SyntaxError("Rule is not callable")
         return rule
     ##
     ## Call pyRule
     ##
-    def _call(self,**kwargs):
+    def __call__(self,**kwargs):
         t=datetime.datetime.now()
         # Try to get compiled rule from cache
         with self.compiled_lock:
@@ -260,7 +348,7 @@ class PyRule(models.Model):
             rule=PyRule.objects.get(name=py_rule_name)
         except PyRule.DoesNotExist:
             raise cls.NoPyRule
-        return rule._call(**kwargs)
+        return rule(**kwargs)
 ##
 ## Search patters
 ##
@@ -334,6 +422,7 @@ class RefBook(models.Model):
     ##
     @classmethod
     def search(cls,user,search,limit):
+        from noc.lib.search import SearchResult # Must be inside method to prevent import loops
         for b in RefBook.objects.filter(is_enabled=True):
             field_names=[f.name for f in b.refbookfield_set.order_by("order")]
             for f in b.refbookfield_set.filter(search_method__isnull=False):
@@ -344,8 +433,8 @@ class RefBook(models.Model):
                 for r in q:
                     text="\n".join(["%s = %s"%(k,v) for k,v in zip(field_names,r.value)])
                     yield SearchResult(
-                        url="/main/refbook/%d/%d/"%(b.id,r.id),
-                        title="Reference Book: %s, column %s"%(b.name,f.name),
+                        url=("main:refbook:item",b.id,r.id),
+                        title=u"Reference Book: %s, column %s"%(b.name,f.name),
                         text=text,
                         relevancy=1.0,
                     )
@@ -449,11 +538,6 @@ class TimePattern(models.Model):
     
     def __unicode__(self):
         return self.name
-    
-    def test_link(self):
-        return "<a href='/main/time_pattern/%d/test/'>Test</a>"%self.id
-    test_link.short_description="Test Time Pattern"
-    test_link.allow_tags=True
     ##
     ## Returns associated Time Pattern object
     ##
@@ -609,6 +693,8 @@ class Notification(models.Model):
     link=models.CharField("Link",max_length=256,null=True,blank=True)
     next_try=models.DateTimeField("Next Try",null=True,blank=True)
     actual_till=models.DateTimeField("Actual Till",null=True,blank=True)
+    def __unicode__(self):
+        return self.subject
 ##
 ## System Notification
 ##
@@ -689,38 +775,3 @@ class UserProfileContact(models.Model):
     time_pattern=models.ForeignKey(TimePattern,verbose_name="Time Pattern")
     notification_method=models.CharField("Method",max_length=16,choices=USER_NOTIFICATION_METHOD_CHOICES)
     params=models.CharField("Params",max_length=256)
-
-##
-## Application Menu
-##
-class AppMenu(Menu):
-    app="main"
-    title="Main"
-    items=[
-        ("Reports",        "/main/report/",           "is_logged_user()"),
-        ("Calculators",    "/main/calculator/",       "is_logged_user()"),
-        ("Audit Trail",    "/admin/main/audittrail/", "is_superuser()"),
-        ("Reference Books", "/main/refbook/",          "is_logged_user()"),
-        ("Browse Data",     "/main/databrowse/",       "is_superuser()"),
-        ("Setup", [
-            ("Users",  "/admin/auth/user/",  "auth.change_user"),
-            ("Groups", "/admin/auth/group/", "auth.change_group"),
-            ("Languages","/admin/main/language/", "main.change_language"),
-            ("MIME Types", "/admin/main/mimetype/", "main.change_mimetype"),
-            ("Configs",    "/main/config/",  "is_superuser()"),
-            ("pyRules",    "/admin/main/pyrule/", "is_superuser()"),
-            ("Reference Books", "/admin/main/refbook/", "main.change_refbook"),
-            ("Time Patterns",   "/admin/main/timepattern/", "main.change_timepattern"),
-            ("Notification Groups",   "/admin/main/notificationgroup/", "main.change_notificationgroup"),
-            ("System Notifications",   "/admin/main/systemnotification/", "main.change_systemnotification"),
-            ("Pending Notifications", "/admin/main/notification/", "main.change_notification"),
-        ]),
-        ("Documentation", [
-            ("Administrator's Guide", "/static/doc/en/ag/html/index.html"),
-            ("User's Guide", "/static/doc/en/ug/html/index.html"),
-        ]),
-    ]
-##
-## Load and register reports
-##
-report_registry.register_all()
