@@ -2,9 +2,11 @@
 import os,sys,site
 # Adjust paths
 d=os.path.dirname(sys.argv[0])
-contrib=os.path.join(d,"contrib","lib")
+if not d:
+    d=os.getcwd()
+contrib=os.path.abspath(os.path.join(d,"contrib","lib"))
 sys.path.insert(0,contrib)
-sys.path.insert(0,os.path.join(d,".."))
+sys.path.insert(0,os.path.abspath(os.path.join(d,"..")))
 sys.path.insert(0,d)
 # Install eggs from contrib/lib
 site.addsitedir(contrib)
@@ -18,4 +20,9 @@ except ImportError:
 from django.core.management import execute_manager
 
 if __name__ == "__main__":
+    if sys.argv[1] not in ["test","syncdb","migrate"]:
+        # Initialize models
+        os.environ["DJANGO_SETTINGS_MODULE"]="noc.settings"
+        import noc.urls
+    # Execute command
     execute_manager(settings)
