@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 ##----------------------------------------------------------------------
-## Copyright (C) 2007-2009 The NOC Project
+## Peer module models
+##----------------------------------------------------------------------
+## Copyright (C) 2007-2010 The NOC Project
 ## See LICENSE for details
 ##----------------------------------------------------------------------
 """
@@ -10,15 +12,15 @@ from noc.settings import config
 from noc.lib.validators import check_asn,check_as_set,is_ipv4,is_cidr
 from noc.lib.tt import tt_url,admin_tt_url
 from noc.lib.rpsl import rpsl_format
-from noc.lib.fields import INETField,InetArrayField
+from noc.lib.fields import INETField,InetArrayField,AutoCompleteTagsField
 from noc.sa.profiles import profile_registry
 from noc.main.models import NotificationGroup
 from noc.cm.models import PrefixList
 from noc.sa.models import AdministrativeDomain
-from noc.main.menu import Menu
-from noc.main.middleware import get_user
+from noc.lib.middleware import get_user
 from noc.lib.fileutils import urlopen
 from noc.lib.crypto import md5crypt
+from noc.lib.app.site import site
 import random,time,logging,urllib,urllib2
 
 ##
@@ -145,10 +147,12 @@ class AS(models.Model):
     header_remarks=models.TextField("Header Remarks",null=True,blank=True) # remarks: will be prepended automatically
     footer_remarks=models.TextField("Footer Remarks",null=True,blank=True) # remarks: will be prepended automatically
     rir=models.ForeignKey(RIR,verbose_name="RIR") # source:
-    def __str__(self):
-        return "AS%d (%s)"%(self.asn,self.description)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
         return u"AS%d (%s)"%(self.asn,self.description)
+    def get_absolute_url(self):
+        return site.reverse("peer:as:change",self.id)
+    
     @classmethod
     def default_as(cls):
         return AS.objects.get(asn=0)
@@ -291,10 +295,11 @@ class ASSet(models.Model):
     members=models.TextField("Members",null=True,blank=True)
     rpsl_header=models.TextField("RPSL Header",null=True,blank=True)
     rpsl_footer=models.TextField("RPSL Footer",null=True,blank=True)
-    def __str__(self):
-        return self.name
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
-        return unicode(self.name)
+        return self.name
+    def get_absolute_url(self):
+        return site.reverse("peer:asset:change",self.id)
     def _member_list(self):
         if self.members is None:
             return []
@@ -439,10 +444,13 @@ class Peer(models.Model):
     max_prefixes=models.IntegerField("Max. Prefixes",default=100)
     import_filter_name=models.CharField("Import Filter Name",max_length=64,blank=True,null=True)
     export_filter_name=models.CharField("Export Filter Name",max_length=64,blank=True,null=True)
-    def __str__(self):
-        return "%s (%s@%s)"%(self.remote_asn,self.remote_ip,self.peering_point.hostname)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
-        return unicode(str(self))
+        return u"%s (%s@%s)"%(self.remote_asn,self.remote_ip,self.peering_point.hostname)
+
+    def get_absolute_url(self):
+        return site.reverse("peer:peer:change",self.id)
+
     def save(self):
         if self.import_filter_name is not None and not self.import_filter_name.strip():
             self.import_filter_name=None
@@ -618,7 +626,7 @@ class WhoisLookup(models.Model):
     def lookup(self,method,query):
         key,value=method.split(":")
         lookup_ids=[l.id for l in WhoisLookup.objects.filter(key=key,value=value)]
-        r=list(WhoisCache.objects.filter(lookup__in=lookup_ids,key=query))
+        r=list(WhoisCache.objects.filter(lookup__in=lookup_ids,key=query.upper()))
         if len(r)==0:
             return set()
         return set(r[0].value.split("|"))
@@ -634,6 +642,12 @@ class WhoisCache(models.Model):
     key=models.CharField("Key",max_length=64)
     value=models.TextField("Value")
     ##
+    ## Convert key to upper case
+    ##
+    def save(self,**kwargs):
+        self.key=self.key.upper()
+        super(WhoisCache,self).save(**kwargs)
+    ##
     ## Fetch data into cache
     ## Returns boolean with update status
     ##
@@ -645,10 +659,10 @@ class WhoisCache(models.Model):
             # Fetch
             urls={}
             for wl in wdb.whoislookup_set.all():
-                if wl.url not in urls:
-                    urls[wl.url]=[wl]
-                else:
+                try:
                     urls[wl.url]+=[wl]
+                except KeyError:
+                    urls[wl.url]=[wl]
             #
             for url in urls:
                 fields=set()
@@ -670,20 +684,24 @@ class WhoisCache(models.Model):
                 data=list(wdb.parse(f,fields))
                 f.close()
                 # Process forward lookups
+                keys=set()
                 for wl in f_set:
                     key=wl.key
                     value=wl.value
                     for d in data:
-                        k=d[key][0]
+                        k=d[key][0].upper()
+                        if k in keys:
+                            continue
                         try:
                             v=d[value]
                         except KeyError:
                             v=[]
                         v=",".join(v)
                         v="|".join([x.strip() for x in v.split(",")])
-                        wc=WhoisCache(lookup=wl,key=k,value=v)
-                        wc.save()
+                        WhoisCache(lookup=wl,key=k,value=v).save()
+                        keys.add(k)
                 # Process reverse lookups
+                keys=set()
                 for wl in r_set:
                     key=wl.key
                     value=wl.value
@@ -705,11 +723,14 @@ class WhoisCache(models.Model):
                             except KeyError:
                                 result[k]=set([v])
                     for key,value in result.items():
-                        wc=WhoisCache(lookup=wl,key=key,value="|".join(value))
-                        wc.save()
+                        key=key.upper()
+                        if key in keys:
+                            continue
+                        WhoisCache(lookup=wl,key=key,value="|".join(value)).save()
+                        keys.add(key)
         return True
 ##
-##
+## Prepared prefix list cache
 ##
 class PrefixListCache(models.Model):
     class Meta:
@@ -723,26 +744,3 @@ class PrefixListCache(models.Model):
     pushed=models.DateTimeField("Pushed",blank=True,null=True)
     def __unicode__(self):
         return u"%s/%s"%(self.peering_point.hostname,self.name)
-
-##
-## Application Menu
-##
-class AppMenu(Menu):
-    app="peer"
-    title="Peering Management"
-    items=[
-        ("ASes",          "/admin/peer/as/"           ,"peer.change_as"),
-        ("ASsets",        "/admin/peer/asset/"        ,"peer.change_asset"),
-        ("Communities",   "/admin/peer/community/"    ,"peer.change_community"),
-        ("Peering Points","/admin/peer/peeringpoint/" ,"peer.change_peeringpoint"),
-        ("Peers",         "/admin/peer/peer/"         ,"peer.change_peer"),
-        ("Prefix List Builder", "/peer/builder/prefix_list/", "peer.change_peer"),
-        ("Setup",[
-            ("Peer Groups",     "/admin/peer/peergroup/"    , "peer.change_peergroup"),
-            ("Community Types", "/admin/peer/communitytype/", "peer.change_communitytype"),
-            ("RIRs",            "/admin/peer/rir/",           "peer.change_rir"),
-            ("Persons",         "/admin/peer/person/",        "peer.change_person"),
-            ("Maintainers",     "/admin/peer/maintainer/"   ,"peer.change_maintainer"),
-            ("Organisations",   "/admin/peer/organisation/"   ,"peer.change_organisation"),
-        ])
-    ]
