@@ -247,6 +247,7 @@ class Activator(Daemon,FSM):
         self.ping_check_results={} # address -> last ping check result
         self.sae_stream=None
         self.event_sources=set()
+        self.ignore_event_rules=[] # [(left_re,right_re)]
         self.trap_collectors=[]   # List of SNMP Trap collectors
         self.syslog_collectors=[] # List of SYSLOG collectors
         self.pm_data_collectors=[] # List of PM Data collectors
@@ -412,11 +413,11 @@ class Activator(Daemon,FSM):
     ##
     ## Script support
     ##
-    def run_script(self,name,access_profile,callback,**kwargs):
-        logging.info("Script %s(%s)"%(name,access_profile.address))
-        pv,pos,sn=name.split(".",2)
+    def run_script(self,script_name,access_profile,callback,**kwargs):
+        logging.info("Script %s(%s)"%(script_name,access_profile.address))
+        pv,pos,sn=script_name.split(".",2)
         profile=profile_registry["%s.%s"%(pv,pos)]()        
-        script=script_registry[name](profile,self,access_profile,**kwargs)
+        script=script_registry[script_name](profile,self,access_profile,**kwargs)
         with self.script_lock:
             self.script_threads[script]=callback
             logging.info("%d script threads"%(len(self.script_threads)))
@@ -593,6 +594,7 @@ class Activator(Daemon,FSM):
                 logging.error("get_event_filter error: %s"%error.text)
                 return
             self.event_sources=set(response.sources)
+            self.compile_ignore_event_rules(response.ignore_rules)
             logging.debug("Setting event source filter to: %s"%str(self.event_sources))
             self.next_filter_update=time.time()+response.expire
         logging.info("Requesting event source filter")
@@ -635,6 +637,11 @@ class Activator(Daemon,FSM):
         r.timestamp=timestamp
         r.ip=ip
         for k,v in body.items():
+            # Check ignore rules
+            for lr,rr in self.ignore_event_rules:
+                if lr.search(k) and rr.search(v):
+                    return # Ignore event
+            # Populate event request
             i=r.body.add()
             i.key=str(k)
             i.value=str(v)
@@ -675,6 +682,17 @@ class Activator(Daemon,FSM):
         self.pm_data_queue=[]
         self.sae_stream.proxy.pm_data(r,pm_data_callback)
     ##
+    ## Compile new ignore event checker
+    ##
+    def compile_ignore_event_rules(self,rules):
+        self.ignore_event_rules=[]
+        for r in rules:
+            try:
+                logging.debug("Adding ignore rule: %s | %s"%(r.left_re,r.right_re))
+                self.ignore_event_rules+=[(re.compile(r.left_re,re.IGNORECASE),re.compile(r.right_re,re.IGNORECASE))]
+            except Exception,why:
+                logging.error("Failed to compile ignore event rule: %s,%s. skipping"%(l,r))
+    ##
     ## Cancel stale scripts
     ##
     @check_state("ESTABLISHED")
