@@ -24,9 +24,9 @@ class InterfaceTypeError(Exception): pass
 class Parameter(object):
     def __init__(self,required=True,default=None):
         self.required=required
-        if default:
-            default=self.clean(default)
-        self.default=default
+        self.default=None
+        if default is not None:
+            self.default=self.clean(default)
     ##
     ## Returns ORParameter
     ##
@@ -35,18 +35,34 @@ class Parameter(object):
     ##
     ##
     ##
-    def raise_error(self,value):
-        raise InterfaceTypeError("%s: %s"%(self.__class__.__name__,repr(value)))
+    def raise_error(self,value,msg=""):
+        raise InterfaceTypeError("%s: %s. %s"%(self.__class__.__name__,repr(value),msg))
     ##
     ## Perform input parameter normalization
     ##
     def clean(self,value):
         self.raise_error(value)
     ##
+    ## Clean up script input parameters.
+    ## Call .clean() by default
+    ##
+    def script_clean_input(self,profile,value):
+        return self.clean(value)
+    ##
+    ## Clean up script result
+    ## Call .clean() by default
+    def script_clean_result(self,profile,value):
+        return self.clean(value)
+    ##
     ## Perform input parameter normalization for form fields
     ##
     def form_clean(self,value):
-        return self.clean(value)
+        if not value and not self.required:
+            return self.default if self.default else None
+        try:
+            return self.clean(value)
+        except InterfaceTypeError,why:
+            raise forms.ValidationError(why)
     ##
     ## Returns an form field object
     ##
@@ -75,15 +91,27 @@ class ORParameter(Parameter):
     InterfaceTypeError: IPParameter: 'xxx'
     """
     def __init__(self,left,right):
+        super(ORParameter,self).__init__()
         self.left=left
         self.right=right
+        
     def clean(self,value):
         try:
-            v=self.left.clean(value)
-            return v
+            return self.left.clean(value)
         except InterfaceTypeError:
-            pass
-        return self.right.clean(value)
+            return self.right.clean(value)
+        
+    def script_clean_input(self,profile,value):
+        try:
+            return self.left.script_clean_input(profile,value)
+        except InterfaceTypeError:
+            return self.right.script_clean_input(profile,value)
+        
+    def script_clean_result(self,profile,value):
+        try:
+            return self.left.script_clean_result(profile,value)
+        except InterfaceTypeError:
+            return self.right.script_clean_result(profile,value)
 ##
 ##
 ##
@@ -112,8 +140,14 @@ class StringParameter(Parameter):
     '10'
     >>> StringParameter().clean(None)
     'None'
+    >>> StringParameter(default="test").clean("no test")
+    'no test'
+    >>> StringParameter(default="test").clean(None)
+    'test'
     """
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         try:
             return str(value)
         except:
@@ -133,12 +167,18 @@ class REStringParameter(StringParameter):
     Traceback (most recent call last):
         ...
     InterfaceTypeError: REStringParameter: 'ex'
+    >>> REStringParameter("ex+p",default="exxp").clean("regexp 1")
+    'regexp 1'
+    >>> REStringParameter("ex+p",default="exxp").clean(None)
+    'exxp'
     """
     def __init__(self,regexp,required=True,default=None):
+        self.rx=re.compile(regexp) # Compile before calling the constructor
         super(REStringParameter,self).__init__(required=required,default=default)
-        self.rx=re.compile(regexp)
 
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         v=super(REStringParameter,self).clean(value)
         if not self.rx.search(v):
             self.raise_error(value)
@@ -164,8 +204,14 @@ class BooleanParameter(Parameter):
     Traceback (most recent call last):
         ...
     InterfaceTypeError: BooleanParameter: []
+    >>> BooleanParameter(default=False).clean(None)
+    False
+    >>> BooleanParameter(default=True).clean(None)
+    True
     """
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         if type(value)==types.BooleanType:
             return value
         if type(value) in [types.IntType,types.LongType]:
@@ -173,6 +219,10 @@ class BooleanParameter(Parameter):
         if type(value) in [types.StringType,types.UnicodeType]:
             return value.lower() in ["true","t","yes","y"]
         self.raise_error(value)
+    ##
+    def get_form_field(self):
+        return forms.BooleanField(required=self.required,initial=self.default)
+    
 ##
 ##
 ##
@@ -194,12 +244,23 @@ class IntParameter(Parameter):
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IntParameter: 10
+    >>> IntParameter(max_value=10,default=7).clean(5)
+    5
+    >>> IntParameter(max_value=10,default=7).clean(None)
+    7
+    >>> IntParameter(max_value=10,default=15)
+    Traceback (most recent call last):
+        ...
+    InterfaceTypeError: IntParameter: 15
     """
     def __init__(self,required=True,default=None,min_value=None,max_value=None):
-        super(IntParameter,self).__init__(required=required,default=default)
         self.min_value=min_value
         self.max_value=max_value
+        super(IntParameter,self).__init__(required=required,default=default)
+        
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         try:
             i=int(value)
         except:
@@ -212,6 +273,8 @@ class IntParameter(Parameter):
 ##
 class ListParameter(Parameter):
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         try:
             return list(value)
         except:
@@ -238,13 +301,32 @@ class InstanceOfParameter(Parameter):
     Traceback (most recent call last):
     ...
     InterfaceTypeError: InstanceOfParameter: 1
+    >>> InstanceOfParameter(cls="C").clean(C()) and "Ok"
+    'Ok'
+    >>> InstanceOfParameter(cls="C").clean(1) and "Ok"
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: InstanceOfParameter: 1
     """
     def __init__(self,cls,required=True,default=None):
         super(InstanceOfParameter,self).__init__(required=required,default=default)
         self.cls=cls
+        if isinstance(cls,basestring):
+            self.is_valid=self.is_valid_classname
+        else:
+            self.is_valid=self.is_valid_instance
+    
+    def is_valid_instance(self,value):
+        return isinstance(value,self.cls)
+    
+    def is_valid_classname(self,value):
+        return hasattr(value,"__class__") and value.__class__.__name__==self.cls
+        
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         try:
-            if isinstance(value,self.cls):
+            if self.is_valid(value):
                 return value
         except:
             pass
@@ -264,18 +346,43 @@ class ListOfParameter(ListParameter):
     InterfaceTypeError: IntParameter: 'x'
     >>> ListOfParameter(element=StringParameter()).clean([1,2,3,"x"])
     ['1', '2', '3', 'x']
+    >>> ListOfParameter(element=StringParameter(),default=[]).clean(None)
+    []
+    >>> ListOfParameter(element=StringParameter(),default=[1,2,3]).clean(None)
+    ['1', '2', '3']
     """
     def __init__(self,element,required=True,default=None):
-        super(ListOfParameter,self).__init__(required=required,default=default)
         self.element=element
+        super(ListOfParameter,self).__init__(required=required,default=default)
         
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         v=super(ListOfParameter,self).clean(value)
         return [self.element.clean(x) for x in v]
+        
+    def script_clean_input(self,profile,value):
+        if value is None and self.default is not None:
+            return self.default
+        v=super(ListOfParameter,self).script_clean_input(profile,value)
+        return [self.element.script_clean_input(profile,x) for x in v]
+
+    def script_clean_result(self,profile,value):
+        if value is None and self.default is not None:
+            return self.default
+        v=super(ListOfParameter,self).script_clean_result(profile,value)
+        return [self.element.script_clean_result(profile,x) for x in v]
+
 ##
 ##
 ##
 class StringListParameter(ListOfParameter):
+    """
+    >>> StringListParameter().clean(["1","2","3"])
+    ['1', '2', '3']
+    >>> StringListParameter().clean(["1",2,"3"])
+    ['1', '2', '3']
+    """
     def __init__(self,required=True,default=None):
         super(StringListParameter,self).__init__(element=StringParameter(),required=required,default=default)
 ##
@@ -294,6 +401,8 @@ class DictParameter(Parameter):
         super(DictParameter,self).__init__(required=required,default=default)
         self.attrs=attrs
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         if type(value)!=types.DictType:
             self.raise_error(value)
         if not self.attrs:
@@ -302,22 +411,69 @@ class DictParameter(Parameter):
         out_value={}
         for a_name,attr in self.attrs.items():
             if a_name not in in_value and attr.required:
-                self.raise_error(value)("Attribute '%s' required"%a_name)
+                self.raise_error(value,"Attribute '%s' is required"%a_name)
             if a_name in in_value:
                 try:
                     out_value[a_name]=attr.clean(in_value[a_name])
-                except InterfaceTypeError:
-                    self.raise_error(value)("Invalid value for '%s'"%a_name)
+                except InterfaceTypeError, why:
+                    self.raise_error(value,"Invalid value for '%s': %s"%(a_name,why))
                 del in_value[a_name]
         for k,v in in_value.items():
             out_value[k]=v
         return out_value
+        
+    def script_clean_input(self,profile,value):
+        if value is None and self.default is not None:
+            return self.default
+        if type(value)!=types.DictType:
+            self.raise_error(value)
+        if not self.attrs:
+            return value
+        in_value=value.copy()
+        out_value={}
+        for a_name,attr in self.attrs.items():
+            if a_name not in in_value and attr.required:
+                self.raise_error(value,"Attribute '%s' required"%a_name)
+            if a_name in in_value:
+                try:
+                    out_value[a_name]=attr.script_clean_input(profile,in_value[a_name])
+                except InterfaceTypeError:
+                    self.raise_error(value,"Invalid value for '%s'"%a_name)
+                del in_value[a_name]
+        for k,v in in_value.items():
+            out_value[k]=v
+        return out_value
+        
+    def script_clean_result(self,profile,value):
+        if value is None and self.default is not None:
+            return self.default
+        if type(value)!=types.DictType:
+            self.raise_error(value)
+        if not self.attrs:
+            return value
+        in_value=value.copy()
+        out_value={}
+        for a_name,attr in self.attrs.items():
+            if a_name not in in_value and attr.required:
+                self.raise_error(value,"Attribute '%s' required"%a_name)
+            if a_name in in_value:
+                try:
+                    out_value[a_name]=attr.script_clean_result(profile,in_value[a_name])
+                except InterfaceTypeError:
+                    self.raise_error(value,"Invalid value for '%s'"%a_name)
+                del in_value[a_name]
+        for k,v in in_value.items():
+            out_value[k]=v
+        return out_value
+
 ##
 ##
 ##
 rx_datetime=re.compile(r"^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(\.\d+)?$")
 class DateTimeParameter(StringParameter):
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         if isinstance(value,datetime.datetime):
             return value.isoformat()
         if value.lower()=="infinite":
@@ -338,6 +494,8 @@ class IPParameter(StringParameter):
     InterfaceTypeError: IPParameter: '192.168.0.256'
     """
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         v=super(IPParameter,self).clean(value)
         X=v.split(".")
         if len(X)!=4:
@@ -369,6 +527,8 @@ class IPv4PrefixParameter(StringParameter):
     InterfaceTypeError: IPParameter: '192.168.0.0/-5'
     """
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         v=super(IPv4PrefixParameter,self).clean(value)
         if "/" not in v:
             self.raise_error(value)
@@ -413,6 +573,7 @@ class VLANIDParameter(IntParameter):
 rx_mac_address_cisco=re.compile("^[0-9A-F]{4}\.[0-9A-F]{4}\.[0-9A-F]{4}$")
 rx_mac_address_cisco_media=re.compile("^01[0-9A-F]{2}\.[0-9A-F]{4}\.[0-9A-F]{4}\.[0-9A-F]{2}$")
 rx_mac_address_sixblock=re.compile("^([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2})$")
+rx_mac_address_hp=re.compile("^[0-9A-F]{6}-[0-9A-F]{6}$")
 class MACAddressParameter(StringParameter):
     """
     >>> MACAddressParameter().clean("1234.5678.9ABC")
@@ -431,6 +592,8 @@ class MACAddressParameter(StringParameter):
     '12:34:56:78:9A:BC'
     >>> MACAddressParameter().clean("0:13:46:50:87:5")
     '00:13:46:50:87:05'
+    >>> MACAddressParameter().clean("123456-789abc")
+    '12:34:56:78:9A:BC'
     >>> MACAddressParameter().clean("12-34-56-78-9A-BC-DE")
     Traceback (most recent call last):
         ...
@@ -441,6 +604,8 @@ class MACAddressParameter(StringParameter):
     InterfaceTypeError: MACAddressParameter: 'AB:CD:EF:GH:HJ:KL'
     """
     def clean(self,value):
+        if value is None and self.default is not None:
+            return self.default
         value=super(MACAddressParameter,self).clean(value)
         value=value.upper()
         match=rx_mac_address_cisco_media.match(value)
@@ -451,17 +616,30 @@ class MACAddressParameter(StringParameter):
         if match:
             value=value.replace(".","")
         else:
-            value=value.replace("-",":")
-            match=rx_mac_address_sixblock.match(value)
-            if not match:
-                self.raise_error(value)
-            value=""
-            for i in range(1,7):
-                v=match.group(i)
-                if len(v)==1:
-                    v="0"+v
-                value+=v
+            match=rx_mac_address_hp.match(value)
+            if match:
+                value=value.replace("-","")
+            else:
+                value=value.replace("-",":")
+                match=rx_mac_address_sixblock.match(value)
+                if not match:
+                    self.raise_error(value)
+                value=""
+                for i in range(1,7):
+                    v=match.group(i)
+                    if len(v)==1:
+                        v="0"+v
+                    value+=v
         return "%s:%s:%s:%s:%s:%s"%(value[:2],value[2:4],value[4:6],value[6:8],value[8:10],value[10:])
+##
+class InterfaceNameParameter(StringParameter):
+    def script_clean_input(self,profile,value):
+        return profile.convert_interface_name(value)
+        
+    def script_clean_result(self,profile,value):
+        return self.script_clean_input(profile,value)
+
+
 ## Stub for interface registry
 interface_registry={}
 
@@ -475,51 +653,80 @@ class InterfaceBase(type):
 ##
 class Interface(object):
     __metaclass__=InterfaceBase
-    def clean(self,**kwargs):
+    ##
+    ## Generator returning (parameter name, parameter instance) pairs
+    ##
+    def gen_parameters(self):
+        for n,p in self.__class__.__dict__.items():
+            if issubclass(p.__class__,Parameter) and n!="returns":
+                yield (n,p)
+    ##
+    ## Clean up all parameters except "returns"
+    ##
+    def clean(self,__profile=None,**kwargs):
         in_kwargs=kwargs.copy()
         out_kwargs={}
-        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
-            if n=="returns":
-                continue
+        for n,p in self.gen_parameters():
             if n not in in_kwargs and p.required:
-                raise InterfaceTypeError("Parameter '%s' required"%n)
+                if p.default is not None:
+                    out_kwargs[n]=p.default
+                else:
+                    raise InterfaceTypeError("Parameter '%s' required"%n)
             if n in in_kwargs:
-                try:
-                    out_kwargs[n]=p.clean(in_kwargs[n])
-                except InterfaceTypeError,why:
-                    raise InterfaceTypeError("Invalid value for '%s': %s"%(n,why))
+                if not (in_kwargs[n] is None and not p.required):
+                    try:
+                        if __profile:
+                            out_kwargs[n]=p.script_clean_input(__profile,in_kwargs[n])
+                        else:
+                            out_kwargs[n]=p.clean(in_kwargs[n])
+                    except InterfaceTypeError,why:
+                        raise InterfaceTypeError("Invalid value for '%s': %s"%(n,why))
                 del in_kwargs[n]
         # Copy other parameters
         for k,v in in_kwargs.items():
-            out_kwargs[k]=v
+            if k!="__profile":
+                out_kwargs[k]=v
         return out_kwargs
+    ##
+    ## Clean up returned result
+    ##
     def clean_result(self,result):
         try:
             rp=self.returns
         except AttributeError:
             return result # No return result restriction
         return rp.clean(result)
+    ##
+    ## Clean up script input
+    ##
+    def script_clean_input(self,__profile,**kwargs):
+        return self.clean(__profile=__profile,**kwargs)
+    ##
+    def script_clean_result(self,__profile,result):
+        try:
+            rp=self.returns
+        except AttributeError:
+            return result
+        return rp.script_clean_result(__profile,result)
+    ##
     def requires_input(self):
-        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
-            if n=="returns":
-                continue
+        for n,p in self.gen_parameters():
             return True
         return False
+        
     def get_form(self,data=None):
         def get_clean_field_wrapper(form,name,param):
             def clean_field_wrapper(form,name,param):
                 data=form.cleaned_data[name]
                 if not param.required and not data:
-                    return data
+                    return None
                 try:
                     return param.form_clean(data)
                 except InterfaceTypeError:
                     raise forms.ValidationError("Invalid value")
             return lambda: clean_field_wrapper(form,name,param)
         f=forms.Form(data)
-        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
-            if n=="returns":
-                continue
+        for n,p in self.gen_parameters():
             f.fields[n]=p.get_form_field()
             setattr(f,"clean_%s"%n,get_clean_field_wrapper(f,n,p))
         return f
