@@ -8,12 +8,12 @@
 from base import *
 
 class IGetMACAddressTable(Interface):
-    interface=REStringParameter(r"^\S+$",required=False)
+    interface=InterfaceNameParameter(required=False)
     vlan=VLANIDParameter(required=False)
     mac=MACAddressParameter(required=False)
     returns=ListOfParameter(element=DictParameter(attrs={"vlan_id"    : VLANIDParameter(),
                                                          "mac"        : MACAddressParameter(),
-                                                         "interfaces" : StringListParameter(),
+                                                         "interfaces" : ListOfParameter(element=InterfaceNameParameter()),
                                                          "type"       : StringParameter(), # choices=["D","S"]
                                                          }))
     
