@@ -9,7 +9,7 @@ import noc.sa.script
 from noc.sa.interfaces import IGetMACAddressTable
 import re
 
-rx_line=re.compile(r"^(?:\*\s+)?(?P<vlan_id>\d+)\s+(?P<mac>\S+)\s+(?P<type>\S+)\s+\S+\s+(?:\S+\s+)?(?P<interfaces>.*)$")
+rx_line=re.compile(r"^(?:\*\s+)?(?P<vlan_id>\d+)\s+(?P<mac>\S+)\s+(?P<type>\S+)\s+(?:\S+\s+){0,2}(?P<interfaces>.*)$")
 
 class Script(noc.sa.script.Script):
     name="Cisco.IOS.get_mac_address_table"
@@ -22,16 +22,15 @@ class Script(noc.sa.script.Script):
             cmd+=" interface %s"%interface
         if vlan is not None:
             cmd+=" vlan %s"%vlan
-        self.cli("terminal length 0")
         vlans=self.cli(cmd)
         r=[]
-        for l in vlans.split("\n"):
+        for l in vlans.splitlines():
             match=rx_line.match(l.strip())
             if match:
                 r.append({
                     "vlan_id"   : match.group("vlan_id"),
                     "mac"       : match.group("mac"),
                     "interfaces": [match.group("interfaces")],
-                    "type"      : {"dynamic":"D","static":"S"}[match.group("type")],
+                    "type"      : {"dynamic":"D","static":"S"}[match.group("type").lower()],
                 })
         return r
