@@ -19,10 +19,14 @@ class Profile(noc.sa.profiles.Profile):
     command_more=" "
     command_enter_config="configure"
     command_leave_config="commit and-quit"
-    pattern_lg_as_path_list=r"(?<=AS path: )(\d+(?: \d+)*)"
-    pattern_lg_best_path=r"^(\s+[+*].+?\s+Router ID: \S+)"
     
     def generate_prefix_list(self,name,pl,strict=True):
+        """
+        >>> Profile().generate_prefix_list("test",["192.168.0.0/24","192.168.1.0/24"])
+        'term pass {\\n    from {\\n        route-filter 192.168.0.0/24 exact;\\n        route-filter 192.168.1.0/24 exact;\\n    }\\n    then next policy;\\n}\\nterm reject {\\n    then reject;\\n}'
+        >>> Profile().generate_prefix_list("test",["192.168.0.0/24","192.168.1.0/24"],strict=False)
+        'term pass {\\n    from {\\n        route-filter 192.168.0.0/24 orlonger;\\n        route-filter 192.168.1.0/24 orlonger;\\n    }\\n    then next policy;\\n}\\nterm reject {\\n    then reject;\\n}'
+        """
         if strict:
             p="        route-filter %s exact;"
         else:
