@@ -7,6 +7,7 @@
 """
 import noc.sa.script
 from noc.sa.interfaces import IGetDot11Associations
+from noc.lib.text import strip_html_tags
 import re
 
 rx_mac=re.compile("(?P<mac>[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2})")
@@ -19,7 +20,7 @@ class Script(noc.sa.script.Script):
             v=self.cli("wlctl authe_sta_list")
         elif self.access_profile.scheme==self.HTTP:
             v=self.http.get("/wlclientview.cmd")
-            v=self.strip_html_tags(v)
+            v=strip_html_tags(v)
         else:
             raise Exception("Unsupported access scheme")
         r=[]
