@@ -7,6 +7,8 @@
 """
 from noc.lib.ip import bits_to_netmask
 from noc.lib.registry import Registry
+from noc.lib.ecma48 import strip_control_sequences
+from noc.sa.interfaces import InterfaceTypeError
 import re
 ##
 ##
@@ -60,25 +62,19 @@ class Profile(object):
     pattern_unpriveleged_prompt=None
     # Regular expression to catch pager
     # (Used in command results)
+    # If pattern_more is string, send command_more
+    # If pattern_more is a list of (pattern,command)
+    # send appropriative command
     pattern_more="^---MORE---"
-    # Regular expression to catch first pager occurence.
-    # (If differ from pattern_more)
-    pattern_more_start=None
-    # Regular expression to catch last pager occurence.
-    # (If differ from pattern_more)
-    pattern_more_end=None
+    # Sequence to be send to list forward pager
+    # If pattern_more is string and is matched
+    command_more="\n"
     # Sequence to be send at the end of all CLI commands
     #
     command_submit="\n"
-    # Sequence to be send to list forward pager
+    # Sequence to disable pager
     #
-    command_more="\n"
-    # Sequence to be send to list pager at first occurence.
-    # (If set to None command_more used)
-    command_more_start=None
-    # Sequence to be send to quit pager when paged to the end
-    # (If set to None command_more used)
-    command_more_end=None
+    command_disable_pager=None
     # Sequence to gracefully close session
     #
     command_exit="exit"
@@ -98,27 +94,9 @@ class Profile(object):
     # before checking any regular expressions
     # (when Action.CLEAN_INPUT==True)
     rogue_chars=["\r"]
-    ##
-    ## Looking glass hilighting support
-    ##
-    
-    # match.group(1) contains AS list <!> Deprecated
-    pattern_lg_as_path_list=None
-    # match.group(1) containts Best Path <!> Deprecated
-    pattern_lg_best_path=None
     # Does the equipment supports bitlength netmasks
     # or netmask should be converted to traditional formats
     requires_netmask_conversion=False
-    
-    #
-    # AS Path hilighting
-    # Deprecated <!>
-    # 
-    def lg_as_path(self,m):
-        def whois_formatter(q):
-            return "<A HREF='http://www.db.ripe.net/whois?AS%s'>%s</A>"%(q,q)
-        as_list=m.group(1).split()
-        return " ".join([whois_formatter(x) for x in as_list])
     #
     # Converts ipv4 prefix to the format acceptable by router
     #
@@ -150,7 +128,31 @@ class Profile(object):
     # Can be changed in derived classes
     #
     convert_mac=convert_mac_to_colon
-    
+    #
+    # Interface name normalization
+    #
+    # Dumb translation
+    def convert_interface_name(self,s):
+        return s
+    # Cisco-like translation
+    rx_cisco_interface_name=re.compile(r"^(?P<type>[a-z]{2})[a-z\-]*\s*(?P<number>\d+(/\d+(/\d+)?)?(\.\d+(\.\d+)?)?)$",re.IGNORECASE)
+    def convert_interface_name_cisco(self,s):
+        """
+        >>> Profile().convert_interface_name_cisco("Gi0")
+        'Gi 0'
+        >>> Profile().convert_interface_name_cisco("GigabitEthernet0")
+        'Gi 0'
+        >>> Profile().convert_interface_name_cisco("Gi 0")
+        'Gi 0'
+        >>> Profile().convert_interface_name_cisco("tengigabitethernet 1/0/1")
+        'Te 1/0/1'
+        >>> Profile().convert_interface_name_cisco("tengigabitethernet 1/0/1.5")
+        'Te 1/0/1.5'
+        """
+        match=self.rx_cisco_interface_name.match(s)
+        if not match:
+            raise InterfaceTypeError("Invalid interface '%s'"%s)
+        return "%s %s"%(match.group("type").capitalize(),match.group("number"))
     #
     # Configuration generators
     #
@@ -171,6 +173,13 @@ class Profile(object):
     #
     config_volatile=None
     #
+    # Preprocessor to clean up and normalize input from device.
+    # Delete ASCII sequences by default.
+    # Can be overriden to achieve desired behavior
+    #
+    def cleaned_input(self,input):
+        return strip_control_sequences(input)
+    #
     # Clean up config
     #
     def cleaned_config(self,cfg):
