@@ -16,7 +16,10 @@ class Profile(noc.sa.profiles.Profile):
     supported_schemes=[SSH]
     pattern_username="^([Uu]sername|[Ll]ogin):"
     pattern_prompt="bp>"
-    pattern_more=r"^(/var/tmp/shell\.out\.\S+|:)"
-    pattern_more_end=r"^\(END\)"
-    command_more="w1000000\n"
-    command_more_end="q"
+    pattern_more=[
+            (r"^(/var/tmp/shell\.out\.\S+|:)"," "),
+            (r"^\(END\) ","q"),
+            ]
+    def cleaned_input(self,input):
+        input=input.replace("\x1b[24;1H\x1b[K","\n")
+        return super(Profile,self).cleaned_input(input)
