@@ -9,7 +9,7 @@
 """
 from __future__ import with_statement
 from noc.sa.models import Activator, ManagedObject, TaskSchedule, MapTask, periodic_registry
-from noc.fm.models import Event,EventData,EventPriority,EventClass,EventCategory
+from noc.fm.models import Event,EventData,EventPriority,EventClass,EventCategory,IgnoreEventRules
 from noc.sa.rpc import RPCSocket,file_hash,get_digest,get_nonce
 from noc.pm.models import TimeSeries
 import logging,time,threading,datetime,os,random,xmlrpclib,cPickle
@@ -106,7 +106,9 @@ class Service(SAEService):
             u.name=n
             u.code=read_file(n)
         done(controller,response=r)
-        
+    ##
+    ## Retrieve event filters
+    ##
     def event_filter(self,controller,request,done):
         if not controller.stream.is_authenticated:
             e=Error()
@@ -117,8 +119,14 @@ class Service(SAEService):
         activator=self.get_controller_activator(controller)
         r=EventFilterResponse()
         r.expire=self.sae.config.getint("sae","refresh_event_filter")
+        # Build source filter
         for c in ManagedObject.objects.filter(activator=activator,trap_source_ip__isnull=False):
             r.sources.append(c.trap_source_ip)
+        # Build event filter
+        for ir in IgnoreEventRules.objects.filter(is_active=True):
+            i=r.ignore_rules.add()
+            i.left_re=ir.left_re
+            i.right_re=ir.right_re
         done(controller,response=r)
         
     def event(self,controller,request,done):
@@ -182,6 +190,14 @@ class SAESocket(RPCSocket,AcceptedTCPSocket):
     @classmethod
     def check_access(cls,address):
         return Activator.check_ip_access(address)
+    
+    def close(self):
+        # Rollback all active transactions
+        e=Error()
+        e.code=ERR_ACTIVATOR_LOST
+        e.text="Connection with activator lost"
+        self.transactions.rollback_all_transactions(e) # Close all active transactions
+        super(AcceptedTCPSocket,self).close()
 ##
 ## SSL version of SAE socket
 ##
@@ -195,6 +211,14 @@ class SAESSLSocket(RPCSocket,AcceptedTCPSSLSocket):
     @classmethod
     def check_access(cls,address):
         return Activator.check_ip_access(address)
+
+    def close(self):
+        # Rollback all active transactions
+        e=Error()
+        e.code=ERR_ACTIVATOR_LOST
+        e.text="Connection with activator lost"
+        self.transactions.rollback_all_transactions(e) # Close all active transactions
+        super(AcceptedTCPSSLSocket,self).close()
     
 ##
 ## XML-RPC support
@@ -505,16 +529,16 @@ class SAE(Daemon):
             r+=[(a.name,s)]
         return r
     
-    def script(self,object,name,callback,**kwargs):
+    def script(self,object,script_name,callback,**kwargs):
         def script_callback(transaction,response=None,error=None):
             if error:
-                logging.error("script(%s,**%s) failed: %s"%(name,kwargs,error.text))
+                logging.error("script(%s,**%s) failed: %s"%(script_name,kwargs,error.text))
                 callback(error=error)
                 return
             result=response.result
             result=cPickle.loads(str(result)) # De-serialize
             callback(result=result)
-        logging.info("script %s(%s)"%(name,object))
+        logging.info("script %s(%s)"%(script_name,object))
         try:
             stream=self.get_activator_stream(object.activator.name)
         except:
@@ -525,7 +549,7 @@ class SAE(Daemon):
             callback(error=e)
             return
         r=ScriptRequest()
-        r.script=name
+        r.script=script_name
         r.access_profile.profile           = object.profile_name
         r.access_profile.scheme            = object.scheme
         r.access_profile.address           = object.address
@@ -614,9 +638,14 @@ class SAE(Daemon):
                     variation=10
                     timeout=random.randint(-timeout/variation,timeout/variation)
                     next_try=datetime.datetime.now()+datetime.timedelta(seconds=timeout)
-                    if next_try<mt.task.stop_time: # Check we're still in task time
+                    if error.code==ERR_OVERLOAD:
+                        next_retries=mt.retries_left
+                    else:
+                        next_retries=mt.retries_left-1
+                    if mt.retries_left and next_try<mt.task.stop_time: # Check we're still in task time and have retries left
                         logging.debug("Retry task: %d"%mt_id)
                         mt.next_try=next_try
+                        mt.retries_left=next_retries
                         mt.status="W"
                         mt.save()
                         return
