@@ -7,12 +7,13 @@
 """
 from __future__ import with_statement
 from noc.lib.fsm import StreamFSM
-from noc.lib.ecma48 import strip_control_sequences
 from noc.lib.registry import Registry
 from noc.lib.nbsocket import PTYSocket,UDPSocket,SocketTimeoutError
 from noc.lib.debug import format_frames,get_traceback_frames
+from noc.lib.text import replace_re_group
 from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP
 from noc.sa.profiles import profile_registry
+from noc.settings import config
 import logging,re,threading,Queue,urllib,httplib,random,base64,hashlib,cPickle,sys,time,datetime
 
 try:
@@ -43,6 +44,8 @@ class ScriptProxy(object):
         self._parent=parent
     def __getattr__(self,name):
         return ScriptCallProxy(self._parent,self._parent.profile.scripts[name])
+    def has_script(self,name):
+        return self._parent.profile.scripts.has_key(name)
 
 class ScriptCallProxy(object):
     def __init__(self,parent,script):
@@ -157,6 +160,7 @@ class Script(threading.Thread):
         self.kwargs=kwargs
         self.scripts=ScriptProxy(self)
         self.need_to_save=False
+        self.to_disable_pager=not self.parent and self.profile.command_disable_pager
         self.log_cli_sessions_path=None # Path to log CLI session
         if self.parent:
             self.log_cli_sessions_path=self.parent.log_cli_sessions_path
@@ -206,12 +210,12 @@ class Script(threading.Thread):
         self.debug("Guarded run")
         # Enforce interface type checking
         for i in self.implements:
-            self.kwargs=i.clean(**self.kwargs)
+            self.kwargs=i.script_clean_input(self.profile,**self.kwargs)
         # Calling script body
         result=self.execute(**self.kwargs)
         # Enforce interface result checking
         for i in self.implements:
-            result=i.clean_result(result)
+            result=i.script_clean_result(self.profile,result)
         self.debug("Script returns with result: %s"%result)
         return result
         
@@ -271,17 +275,31 @@ class Script(threading.Thread):
             self.cli_provider=s_class(self.activator.factory,self.profile,self.access_profile)
             self.cli_queue_get()
             self.debug("CLI Provider is ready")
+            # Disable pager when necessary
+            if self.to_disable_pager:
+                self.debug("Disable paging")
+                self.to_disable_pager=False
+                self.cli(self.profile.command_disable_pager)
         return self.cli_provider
         
     def cli(self,cmd,command_submit=None,bulk_lines=None):
+        #
         self.debug("cli(%s)"%cmd)
         self.cli_debug(cmd,">")
         self.request_cli_provider()
-        self.cli_provider.submit(cmd,command_submit=self.profile.command_submit if command_submit is None else command_submit,bulk_lines=bulk_lines)
+        # Submit command
+        command_submit=self.profile.command_submit if command_submit is None else command_submit
+        self.cli_provider.submit(cmd,command_submit=command_submit,bulk_lines=bulk_lines)
         data=self.cli_queue_get()
         if self.strip_echo and data.lstrip().startswith(cmd):
-            data=self.strip_first_lines(data.lstrip())
-        self.debug("cli() returns:\n---------\n%s\n---------"%repr(data))
+            data=data.lstrip()
+            if data.startswith(cmd+"\n"):
+                # Remove first line
+                data=self.strip_first_lines(data.lstrip())
+            else:
+                # Some switches, like ProCurve do not send \n after the echo
+                data=data[len(cmd):]
+        self.debug("cli(%s) returns:\n---------\n%s\n---------"%(cmd,repr(data)))
         self.cli_debug(data,"<")
         return data
     ##
@@ -319,6 +337,11 @@ class Script(threading.Thread):
                 result[int(x)]=None
         return sorted(result.keys())
     ##
+    ## Convert a 6-octet string to MAC address
+    ##
+    def hexstring_to_mac(self,s):
+        return ":".join(["%02X"%ord(x) for x in s])
+    ##
     ## Cancel script
     ##
     def cancel_script(self):
@@ -349,6 +372,12 @@ class Script(threading.Thread):
         self.need_to_save=True
         if self.parent:
             self.parent.schedule_to_save()
+    
+    def motd(self):
+        if not self.cli_provider:
+            self.request_cli_provider()
+        return self.cli_provider.motd
+    motd=property(motd)
 
 ##
 ##
@@ -365,7 +394,8 @@ class CLI(StreamFSM):
             "USERNAME"            : "USERNAME",
             "PASSWORD"            : "PASSWORD",
             "UNPRIVELEGED_PROMPT" : "UNPRIVELEGED_PROMPT",
-            "PROMPT"              : "PROMPT"
+            "PROMPT"              : "PROMPT",
+            "PAGER"               : "START",
         },
         "USERNAME":{
             "USERNAME"            : "FAILURE",
@@ -377,12 +407,14 @@ class CLI(StreamFSM):
             "USERNAME"            : "FAILURE",
             "PASSWORD"            : "FAILURE",
             "UNPRIVELEGED_PROMPT" : "UNPRIVELEGED_PROMPT",
-            "PROMPT"              : "PROMPT"
+            "PROMPT"              : "PROMPT",
+            "PAGER"               : "PASSWORD",
         },
         "SUPER_PASSWORD" : {
             "UNPRIVELEGED_PROMPT" : "FAILURE",
             "PASSWORD"            : "FAILURE",
             "PROMPT"              : "PROMPT",
+            "PAGER"               : "SUPER_PASSWORD"
         },
         "UNPRIVELEGED_PROMPT":{
             "PASSWORD"            : "SUPER_PASSWORD",
@@ -391,8 +423,6 @@ class CLI(StreamFSM):
         "PROMPT":{
             "PROMPT"      : "PROMPT",
             "PAGER"       : "PROMPT",
-            "PAGER_START" : "PROMPT",
-            "PAGER_END"   : "PROMPT",
             "CLOSE"       : "CLOSED",
         },
         "FAILURE":{
@@ -409,15 +439,27 @@ class CLI(StreamFSM):
         self.collected_data=""
         self.submitted_data=[]
         self.submit_lines_limit=None
-        self.prompt_patterns=[x for x in [self.profile.pattern_more,self.profile.pattern_more_start,self.profile.pattern_more_end] if x]
+        self.motd="" # Message of the day storage
+        self.pattern_prompt=None # Set when entering PROMPT state
+        if isinstance(self.profile.pattern_more,basestring):
+            self.more_patterns=[self.profile.pattern_more]
+            self.more_commands=[self.profile.command_more]
+        else:
+            # .more_patterns is a list of (pattern,command)
+            self.more_patterns=[x[0] for x in self.profile.pattern_more]
+            self.more_commands=[x[1] for x in self.profile.pattern_more]
+        self.pager_patterns="|".join([r"(%s)"%p for p in self.more_patterns])
         StreamFSM.__init__(self)
     
     def on_read(self,data):
         self.debug("on_read: %s"%repr(data))
         if self.profile.rogue_chars:
             for rc in self.profile.rogue_chars:
-                data=data.replace(rc,"")
-        self.feed(data,cleanup=strip_control_sequences)
+                try:
+                    data=rc.sub("",data) # rc is compiled regular expression
+                except AttributeError:
+                    data=data.replace(rc,"") # rc is a string
+        self.feed(data,cleanup=self.profile.cleaned_input)
         self.__flush_submitted_data()
     ##
     ## Feed pending submitted data
@@ -437,22 +479,24 @@ class CLI(StreamFSM):
             self.__flush_submitted_data()
         else:
             self.write(msg+(self.profile.command_submit if command_submit is None else command_submit))
-    
+    ##
     def on_START_enter(self):
+        # username/password match
         p=[
             (self.profile.pattern_username,"USERNAME"),
             (self.profile.pattern_password,"PASSWORD"),
         ]
+        # Match unpriveleged prompt when given
         if self.profile.pattern_unpriveleged_prompt:
-            p+=[
-                (self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT"),
-            ]
-        p+=[
-            (self.profile.pattern_prompt,"PROMPT"),
-        ]
+            p+=[(self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT")]
+        # Match priveleged prompt when given
+        p+=[(self.profile.pattern_prompt,"PROMPT")]
+        # Match pager
+        p+=[(self.pager_patterns,"PAGER")]
         self.set_patterns(p)
     
     def on_USERNAME_enter(self):
+        self.motd="" # Reset MoTD
         self.set_patterns([
             (self.profile.pattern_password, "PASSWORD"),
             (self.profile.pattern_prompt,   "PROMPT"),
@@ -460,29 +504,30 @@ class CLI(StreamFSM):
         self.submit(self.access_profile.user)
         
     def on_PASSWORD_enter(self):
+        self.motd="" # Reset MoTD
         p=[(self.profile.pattern_prompt, "PROMPT")]
         if self.profile.pattern_unpriveleged_prompt:
-            p+=[
-                (self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT"),
-            ]
+            p+=[(self.profile.pattern_unpriveleged_prompt,"UNPRIVELEGED_PROMPT")]
         p+=[
             (self.profile.pattern_username, "USERNAME"),
             (self.profile.pattern_password, "PASSWORD")
             ]
+        p+=[(self.pager_patterns,"PAGER")]
         self.set_patterns(p)
         self.submit(self.access_profile.password)
         
     def on_UNPRIVELEGED_PROMPT_enter(self):
         self.set_patterns([
             (self.profile.pattern_prompt,   "PROMPT"),
-            (self.profile.pattern_password, "PASSWORD"),
+            (self.profile.pattern_password, "PASSWORD")
         ])
         self.submit(self.profile.command_super)
     
     def on_SUPER_PASSWORD_enter(self):
         self.set_patterns([
             (self.profile.pattern_prompt, "PROMPT"),
-            (self.profile.pattern_password, "PASSWORD")
+            (self.profile.pattern_password, "PASSWORD"),
+            (self.pager_patterns,"PAGER")
         ])
         sp=self.access_profile.super_password
         if not sp:
@@ -490,44 +535,47 @@ class CLI(StreamFSM):
         self.submit(sp)
         
     def on_PROMPT_enter(self):
+        self.debug("on_PROMPT_enter")
         if not self.is_ready:
+            self.pattern_prompt=self.profile.pattern_prompt
+            # Refine prompt pattern when necessary
+            for k,v in self.match.groupdict().items():
+                v=re.escape(v)
+                self.pattern_prompt=replace_re_group(self.pattern_prompt,"(?P<%s>"%k,v)
+                self.pattern_prompt=replace_re_group(self.pattern_prompt,"(?P=%s"%k,v)
+            self.debug("Using prompt pattern: %s"%self.pattern_prompt)
             self.queue.put(None) # Signal provider passing into PROMPT state
             self.is_ready=True
         p=[
-            (self.profile.pattern_prompt, "PROMPT"),
-            (self.profile.pattern_more,   "PAGER"),
+            (self.pattern_prompt, "PROMPT"),
+            (self.pager_patterns, "PAGER"),
             ]
-        if self.profile.pattern_more_start:
-            p+=[(self.profile.pattern_more_start, "PAGER_START")]
-        if self.profile.pattern_more_end:
-            p+=[(self.profile.pattern_more_end, "PAGER_END")]
         self.set_patterns(p)
         
     def on_PROMPT_match(self,data,match):
-        if match.re.pattern in self.prompt_patterns:
+        if match.re.pattern in self.pager_patterns:
             self.collected_data+=data
-        elif match.re.pattern==self.profile.pattern_prompt:
+        elif match.re.pattern==self.pattern_prompt:
             self.queue.put(self.collected_data+data)
             self.collected_data=""
         
     def on_PROMPT_PAGER(self):
-        self.write(self.profile.command_more)
+        pg=self.match.group(0)
+        for p,c in zip(self.more_patterns,self.more_commands):
+            if re.search(p,pg,re.DOTALL|re.MULTILINE):
+                self.write(c)
+                return
+        raise Exception("Unexpected pager pattern")
+    on_START_PAGER=on_PROMPT_PAGER
+    on_PASSWORD_PAGER=on_PROMPT_PAGER
     
-    def on_PROMPT_PAGER_START(self):
-        if self.profile.command_more_start is None:
-            self.write(self.profile.command_more)
-        else:
-            self.write(self.profile.command_more_start)
-    
-    def on_PROMPT_PAGER_END(self):
-        if self.profile.command_more_end is None:
-            self.write(self.profile.command_more)
-        else:
-            self.write(self.profile.command_more_end)
-
     def on_FAILURE_enter(self):
         self.set_patterns([])
 
+    def on_START_match(self,data,match):
+        self.motd+=data
+    on_USERNAME_match=on_START_match
+    on_PASSWORD_match=on_START_match
 ##
 ##
 ##
@@ -539,7 +587,7 @@ class CLITelnetSocket(ScriptSocket,CLI,PTYSocket):
     TTL=30
     def __init__(self,factory,profile,access_profile):
         logging.debug("CLITelnetSocket connecting '%s'"%access_profile.address)
-        cmd_args=["/usr/bin/telnet",access_profile.address]
+        cmd_args=[config.get("path","telnet"),access_profile.address]
         if access_profile.port and access_profile.port!=23:
             cmd_args+=[str(access_profile.port)]
         CLI.__init__(self,profile,access_profile)
@@ -552,7 +600,7 @@ class CLISSHSocket(ScriptSocket,CLI,PTYSocket):
     TTL=30
     def __init__(self,factory,profile,access_profile):
         logging.debug("CLISSHSocket connecting '%s'"%access_profile.address)
-        cmd_args=["/usr/bin/ssh","-o","StrictHostKeyChecking no","-l",access_profile.user]
+        cmd_args=[config.get("path","ssh"),"-o","StrictHostKeyChecking=no","-o","UserKnownHostsFile=/dev/null","-l",access_profile.user]
         if access_profile.port and access_profile.port!=22:
             cmd_args+=["-p",str(access_profile.port)]
         cmd_args+=[access_profile.address]
@@ -645,6 +693,7 @@ class SNMPGetSocket(UDPSocket):
         self.oid=oid
         self.address=self.provider.access_profile.address
         self.got_result=False
+        self.is_failed=False
         self.sendto(self.get_snmp_request(),(self.address,161))
     ##
     ## Build full community, appending suffix when necessary
@@ -702,21 +751,42 @@ class SNMPGetSocket(UDPSocket):
         if not self.got_result:
             self.provider.script.debug("SNMP Timeout")
             self.provider.queue.put(None)
+        elif self.is_failed:
+            self.provider.script.debug("Stale SNMP Operation")
+            self.provider.queue.put(None)
         super(SNMPGetSocket,self).on_close()
+    ##
+    ## Raise timeout error when in stale
+    ##
+    def is_stale(self):
+        r=super(SNMPGetSocket,self).is_stale()
+        if r:
+            self.is_failed=True
+        return r
 ##
 ##
 ##
 class SNMPGetNextSocket(SNMPGetSocket):
     TTL=5
+    
+    def __init__(self,provider,oid,bulk=False,min_index=None,max_index=None):
+        self.bulk=bulk
+        self.min_index=min_index
+        self.max_index=max_index
+        super(SNMPGetNextSocket,self).__init__(provider,oid)
     ##
     ## Returns string containing SNMP GET requests to self.oids
     ##
     def get_snmp_request(self):
-        self.provider.script.debug("%s SNMP GETNEXT %s"%(self.address,str(self.oid)))
+        self.provider.script.debug("%s SNMP %s %s"%(self.address,"GETBULK" if self.bulk else "GETNEXT",str(self.oid)))
         p_mod=api.protoModules[api.protoVersion2c]
-        req_PDU =  p_mod.GetNextRequestPDU()
-        p_mod.apiPDU.setDefaults(req_PDU)
-        p_mod.apiPDU.setVarBinds(req_PDU,[(p_mod.ObjectIdentifier(self.oid_to_tuple(self.oid)),p_mod.Null())])
+        req_PDU =  p_mod.GetBulkRequestPDU() if self.bulk else p_mod.GetNextRequestPDU()
+        self.api_pdu=p_mod.apiBulkPDU if self.bulk else p_mod.apiPDU
+        self.api_pdu.setDefaults(req_PDU)
+        if self.bulk:
+            self.api_pdu.setNonRepeaters(req_PDU,0)   # <!>
+            self.api_pdu.setMaxRepetitions(req_PDU,20)# <!>
+        self.api_pdu.setVarBinds(req_PDU,[(p_mod.ObjectIdentifier(self.oid_to_tuple(self.oid)),p_mod.Null())])
         req_msg = p_mod.Message()
         p_mod.apiMessage.setDefaults(req_msg)
         p_mod.apiMessage.setCommunity(req_msg, self.get_community())
@@ -726,18 +796,20 @@ class SNMPGetNextSocket(SNMPGetSocket):
         return encoder.encode(req_msg)
 
     def on_read(self,data,address,port):
+        self.update_status()
         p_mod=api.protoModules[api.protoVersion2c]
         while data:
+            self.provider.script.debug("SNMP PDU RECEIVED")
             rsp_msg, data = decoder.decode(data, asn1Spec=p_mod.Message())
             rsp_pdu = p_mod.apiMessage.getPDU(rsp_msg)
             # Match response to request
-            if p_mod.apiPDU.getRequestID(self.req_PDU)==p_mod.apiPDU.getRequestID(rsp_pdu):
+            if self.api_pdu.getRequestID(self.req_PDU)==p_mod.apiPDU.getRequestID(rsp_pdu):
                 # Check for SNMP errors reported
-                errorStatus = p_mod.apiPDU.getErrorStatus(rsp_pdu)
+                errorStatus = self.api_pdu.getErrorStatus(rsp_pdu)
                 if errorStatus and errorStatus != 2:
                     raise errorStatus
                 # Format var-binds table
-                var_bind_table = p_mod.apiPDU.getVarBindTable(self.req_PDU, rsp_pdu)
+                var_bind_table = self.api_pdu.getVarBindTable(self.req_PDU, rsp_pdu)
                 # Report SNMP table
                 for table_row in var_bind_table:
                     for name, val in table_row:
@@ -746,8 +818,23 @@ class SNMPGetNextSocket(SNMPGetSocket):
                         oid=name.prettyPrint()
                         if not oid.startswith(self.oid):
                             self.close()
+                            self.provider.queue.put(None)
                             return
-                        self.provider.script.debug('%s SNMP GETNEXT REPLY: %s %s'%(self.address,oid,str(val)))
+                        # Check index range if given
+                        if self.min_index is not None or self.max_index is not None:
+                            index=int(oid.split(".")[-1])
+                            if self.min_index is not None and index<self.min_index:
+                                # Skip values below min_index
+                                continue
+                            if self.max_index and index>self.max_index:
+                                # Finish processing
+                                self.close()
+                                self.provider.queue.put(None)
+                                return
+                        if self.bulk:
+                            self.provider.script.debug("SNMP BULK DATA: %s %s"%(oid,str(val)))
+                        else:
+                            self.provider.script.debug('%s SNMP GETNEXT REPLY: %s %s'%(self.address,oid,str(val)))
                         self.provider.queue.put((oid,str(val)))
                         self.got_result=True
                 # Stop on EOM
@@ -757,9 +844,8 @@ class SNMPGetNextSocket(SNMPGetSocket):
                     else:
                         self.close()
                         return
-                # Generate request for next row
-                p_mod.apiPDU.setVarBinds(self.req_PDU, map(lambda (x,y),n=p_mod.Null(): (x,n), var_bind_table[-1]))
-                p_mod.apiPDU.setRequestID(self.req_PDU, p_mod.getNextRequestID())
+                self.api_pdu.setVarBinds(self.req_PDU, map(lambda (x,y),n=p_mod.Null(): (x,n), var_bind_table[-1]))
+                self.api_pdu.setRequestID(self.req_PDU, p_mod.getNextRequestID())
                 self.sendto(encoder.encode(self.req_msg),(self.address,161))
 ##
 ##
@@ -790,24 +876,46 @@ class SNMPProvider(object):
     ## for oid,v in self.getnext("xxxxx"):
     ##      ....
     ##
-    def getnext(self,oid,community_suffix=None):
+    def getnext(self,oid,community_suffix=None,bulk=False,min_index=None,max_index=None):
         self.community_suffix=community_suffix
         if self.getnext_socket:
             self.getnext_socket.close()
-        self.getnext_socket=SNMPGetNextSocket(self,oid)
+        self.getnext_socket=SNMPGetNextSocket(self,oid,bulk=bulk,min_index=min_index,max_index=max_index)
         # Flush queue
         while not self.queue.empty():
             self.queue.get()
         while True:
             r=self.queue.get(block=True)
             if r is None:
-                if self.getnext_socket.got_result: # Stop Iteration in case of success
+                if self.getnext_socket.is_failed:  # Stale socket killed
+                    raise self.TimeOutError()
+                elif self.getnext_socket.got_result: # Stop Iteration in case of success
                     raise StopIteration
                 else: # Socket closed by Timeout
                     raise self.TimeOutError()
             else:
                 yield r
     ##
+    ## getnext wrapper
+    ## returns a hash of {index,value}
+    ##
+    def get_table(self,oid,community_suffix=None,bulk=False):
+        r={}
+        for o,v in self.getnext(oid,community_suffix=community_suffix,bulk=bulk):
+            r[int(o.split(".")[-1])]=v
+        return r
+    ##
+    ## Generator returning a rows of two snmp tables joined by index
+    ##
+    def join_tables(self,oid1,oid2,community_suffix=None,bulk=False):
+        t1=self.get_table(oid1,community_suffix=community_suffix,bulk=bulk)
+        t2=self.get_table(oid2,community_suffix=community_suffix,bulk=bulk)
+        for k1,v1 in t1.items():
+            try:
+                yield (v1,t2[k1])
+            except KeyError:
+                pass
+    ##
     ## Close all UDP sockets
     ##
     def close(self):
